{thread that runs in the background & emails any new audio files it finds}
unit MailerThread;

interface

uses
  Classes, Windows, SysUtils, inifiles, NMsmtp,dialogs;

type
  TMailer = class(TThread)
  private
   extension: string;
   waveCount: integer;
   emailto,emailfrom,emailsubject,emailbody,emailhost : string;
   emailport : integer;
   checkfileinterval: integer;
  protected
    procedure Execute; override;
    procedure sendSMTPMessage(filename: string);
    function LPad(s: String; nLength: integer): string ;
    procedure log(msg: string);
  public
    constructor create(IniFile: TIniFile; _waveCount: integer; usingMP3: boolean);
    destructor destroy; override;
  end;

implementation




{constructor}
constructor TMailer.create(IniFile: TIniFile; _waveCount: integer; usingMP3: boolean);
begin
   inherited create(true); // create but don't start running yet

   if usingMP3 then
      extension := 'mp3'
   else
      extension := 'wav';

   waveCount := _waveCount+1;
   with IniFile do
   begin
      checkfileinterval := ReadInteger('main', 'email.checkFileInterval', 500);
      emailto := ReadString('main', 'email.to', 'you@mycompany.com');
      emailfrom := ReadString('main', 'email.from', 'me@mycompany.com');
      emailsubject := ReadString('main', 'email.subject', 'Sound detected');
      emailbody := ReadString('main', 'email.body', 'Sound was detected, audio file attached!');
      emailhost := ReadString('main', 'email.host', 'localhost');
      emailport := ReadInteger('main', 'email.port', 25);
   end;
end;

{destructor}
destructor TMailer.Destroy;
begin
   inherited destroy;
end;


{main thread entry point}
procedure TMailer.Execute;
var
   filename: string;
begin
   // this thread looks for the next audio file number, and mails it when found
   repeat
      filename := 'rec_' + lpad(inttostr(waveCount), 6) + '.' + extension;
      if FileExists(filename) then
      begin
         sendSMTPMessage(filename);
         inc(waveCount);
      end
      else
      begin
         sleep(checkfileinterval);
      end;
   until terminated;
end;

procedure TMailer.sendSMTPMessage(filename: string);
var
   smtp: TNMSMTP;
   toAddress, body, attachments: TStringList;
begin
   // connect to the SMTP server, send a file and disconnect
   // it would probably be more efficient to only make a single connection and send a whole bunch
   // of files at once, but I'm not sure if some smtp servers will allow this, so I'm keeping it simple
   try

   smtp := TNMSMTP.create(nil);
   toAddress := TStringList.create;
   body := TStringList.create;
   attachments := TStringList.create;
   try
      with smtp do
      begin
         host := emailhost;
         port := emailport;
         timeout := 60000;
         with PostMessage do
         begin
            body.add(emailbody);
            subject := emailsubject + ' - ' + filename;
            toAddress.add(emailto);
            fromName := emailfrom;
            fromAddress := emailfrom;
            replyto := emailfrom;
            attachments.add(filename);
         end;
         connect;
         sendMail;
         disconnect;
      end;
   finally
      attachments.free;
      toAddress.free;
      body.free;
      smtp.free;
   end;

   except
      on e:exception do
      begin
         log(e.Message);
      end;
   end;
end;

{left-pads a string}
function TMailer.LPad(s: String; nLength: integer): string ;
begin
     while length(s) < nLength do
           s := '0' + s ;
     result := s ;
end;

procedure TMailer.log(msg: string);
const
   logFileName = 'smtperrors.log';
var
   logfile: text;
begin
   try
      assignfile(logfile, logfilename);
      if fileexists(logfilename) then
         append(logfile)
      else
         rewrite(logfile);

      writeln(logfile, DateTimeToStr(Now) + ' ' + msg);
      closefile(logfile);
   except
      on E:Exception do
      begin
         // now what?
      end;
   end;
end;

end.
